/*
 *   kwrl - a little VRML 2.0 editor
 *   Copyright (C) 1998,99  Mark R. Stevens
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* local headers */
#include <SFToken.h>
#include <SFString.h>
#include <SFEvent.h>
#include <CylinderSensor.h>
#include <Children.h>

/*************************************************************************/
void CylinderSensor::eventIn(SFString &eventType, SFNode *eventData)
{

#ifdef DEBUG_EVENTS
  cout << "\nEntered CylinderSensor::eventsIn (" << DEF << ")\n";
#endif 

  /* Determine what action to take based on incoming event */
  if ((eventType == "minAngle") || (eventType == "set_minAngle")) {
    SFFloat *n = (SFFloat *) eventData;
    minAngle = *n;

  } else if ((eventType == "maxAngle") || (eventType == "set_maxAngle")) {
    SFFloat *n = (SFFloat *) eventData;
    maxAngle = *n;

  } else if ((eventType == "enabled") || (eventType == "set_enabled")) {
    SFBool *n = (SFBool *) eventData;
    enabled = *n;

  } else if ((eventType == "diskAngle") || (eventType == "set_diskAngle")) {
    SFFloat *n = (SFFloat *) eventData;
    diskAngle = *n;

  } else if ((eventType == "offset") || (eventType == "set_ofset")) {
    SFFloat *n = (SFFloat *) eventData;
    offset = *n;

  } else if ((eventType == "autoOffset") || (eventType == "set_autoOffset")) {
    SFBool *n = (SFBool *) eventData;
    autoOffset = *n;

  }


}
/*************************************************************************/

/*************************************************************************/
void CylinderSensor::eventOut(SFString &, SFNode *)
{

#ifdef DEBUG_EVENTS
  cout << "\nEntered CylinderSensor::eventsOut (" << DEF << ")\n";
#endif 
  
}
/*************************************************************************/

/*************************************************************************/
void CylinderSensor::parse(char *header, istream &InFile)
{

  /* we are valid */
  isValid() = true;

  /* a token in the file */
  SFToken Token;

  /* the required labels */
  bool TokenFound  = false;
  bool BeginBracketFound   = false;

  /* commence to search for the matching bracket */
  while (! InFile.eof()) {
    
    /* Remember where the token started from */
    Token.GetToken(InFile);

#ifdef DEBUG_PARSE
    cout << "\tCylinderSensor: (" << Token() << ")\n";
#endif

    /* based on what token was found, continue parsing */
    if (Token == "DEF") {
      Token.GetToken(InFile);
      DEF = Token();

    } else if (Token == "USE") {
      Token.GetToken(InFile);
      USE = Token();
      unsatisfiedUSE() = true;
      return;

    } else if (Token == "minAngle") {
      minAngle.parse(header, InFile);

    } else if (Token == "maxAngle") {
      maxAngle.parse(header, InFile);

    } else if (Token == "enabled") {
      enabled.parse(header, InFile);

    } else if (Token == "diskAngle") {
      diskAngle.parse(header, InFile);

    } else if (Token == "offset") {
      offset.parse(header, InFile);

    } else if (Token == "autoOffset") {
      autoOffset.parse(header, InFile);

    } else if (Token == "CylinderSensor") {
      TokenFound = true;

    } else if (Token == "{") {
      BeginBracketFound = true;

    } else if (Token == "}") {
      break;
      
    } else {
      parseWarning(Token());

    }
  }

  /* if we did not find the material token we are in trouble */
  if (TokenFound == false) {
    cerr << "\nError:\n";
    cerr << "\tOccurred in (" << nodeType() << "::parse())\n";
    cerr << "\tDid not find expected identifier token.\n";
    exit(0);
  }
  if (BeginBracketFound == false) {
    cerr << "\nError:\n";
    cerr << "\tOccurred in (" << nodeType() << "::parse())\n";
    cerr << "\tDid not find expected \"{\" token.\n";
    exit(0);
  }
}
/*************************************************************************/

