/*
 *   kwrl - a little VRML 2.0 editor
 *   Copyright (C) 1998,99  Mark R. Stevens
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* local headers */
#include <SFToken.h>
#include <NavigationInfo.h>

/*************************************************************************/
void NavigationInfo::eventIn(SFString &eventType, SFNode *eventData)
{

#ifdef DEBUG_EVENTS
  cout << "\nEntered NavigationInfo::eventsIn (" << DEF << ")\n";
#endif 

  /* Determine what action to take based on incoming event */
  if ((eventType == "avatarSize") || (eventType == "set_avatarSize")) {
    MFFloat *n = (MFFloat *) eventData;
    avatarSize = *n;

  } else if ((eventType == "headlight") || (eventType == "set_headlight")) {
    SFBool *n = (SFBool *) eventData;
    headlight = *n;

  } else if ((eventType == "speed") || (eventType == "set_speed")) {
    SFFloat *n = (SFFloat *) eventData;
    speed = *n;

  } else if ((eventType == "type") || (eventType == "set_type")) {
    MFString *n = (MFString *) eventData;
    type = *n;

  } else if ((eventType == "visibilityLimit") || (eventType == "set_visibilityLimit")) {
    SFFloat *n = (SFFloat *) eventData;
    visibilityLimit = *n;

  } else if (eventType == "set_bind") {
    SFBool *n = (SFBool *) eventData;
    isBound = *n;
    SFString out = "isBound";
    eventOut(out, &(isBound));

  }
}
/*************************************************************************/

/*************************************************************************/
void NavigationInfo::eventOut(SFString &, SFNode *)
{

#ifdef DEBUG_EVENTS
  cout << "\nEntered NavigationInfo::eventsOut(" << DEF << ")\n";
#endif 

}
/*************************************************************************/

/*************************************************************************/
void NavigationInfo::parse(char *header, istream &InFile)
{

  /* we are valid */
  isValid() = true;

  /* a token in the file */
  SFToken Token;

  /* the required labels */
  bool TokenFound  = false;
  bool BeginBracketFound   = false;

  /* commence to search for the matching bracket */
  while (! InFile.eof()) {
    
    /* Remember where the token started from */
    Token.GetToken(InFile);
    
#ifdef DEBUG_PARSE    
    cout << "\tNavigationInfo: (" << Token() << ")\n";
#endif
    
    /* based on what token was found, continue parsing */
    if (Token == "DEF") {
      Token.GetToken(InFile);
      DEF = Token();

    } else if (Token == "USE") {
      Token.GetToken(InFile);
      USE = Token();
      unsatisfiedUSE() = true;
      return;

    } else if (Token == "avatarSize") {
      avatarSize.parse(header, InFile);

    } else if (Token == "headlight") {
      headlight.parse(header, InFile);

    } else if (Token == "speed") {
      speed.parse(header, InFile);

    } else if (Token == "visibilityLimit") {
      visibilityLimit.parse(header, InFile);

    } else if (Token == "type") {
      type.parse(header, InFile);

    } else if (Token == "NavigationInfo") {
      TokenFound = true;

    } else if (Token == "{") {
      BeginBracketFound = true;

    } else if (Token == "}") {
      break;
      
    } else {
      parseWarning(Token());

    }
  }

  /* if we did not find the material token we are in trouble */
  if (TokenFound == false) {
    cerr << "\nError:\n";
    cerr << "\tOccurred in (" << nodeType() << "::parse())\n";
    cerr << "\tDid not find expected identifier token.\n";
    exit(0);
  }
  if (BeginBracketFound == false) {
    cerr << "\nError:\n";
    cerr << "\tOccurred in (" << nodeType() << "::parse())\n";
    cerr << "\tDid not find expected \"{\" token.\n";
    exit(0);
  }
}
/*************************************************************************/
