/*
 *   kwrl - a little VRML 2.0 editor
 *   Copyright (C) 1998,99  Mark R. Stevens
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* local headers */
#include <Shape.h>
#include <SFToken.h>

/*************************************************************************/
void Shape::prepare(SFVec3f &min, SFVec3f &max)
{

#ifdef DEBUG_PREPARE
  cout << "\nEntered Shape::prepare (" << DEF << ")\n";
#endif 

  /* prepare the appearance */
  appearance.prepare(min, max);

  /* prepare the geometry */
  geometry.prepare(min, max);
  
}
/*************************************************************************/

/*************************************************************************/
void Shape::render(SFRenderInfo &SFRI)
{

#ifdef DEBUG_RENDER
  cout << "\nEntered Shape::render (" << DEF << ")\n";
  cout << "\t" << gluErrorString(glGetError()) << "\n";  
#endif 

  /* store the polygon flags for restoration */
  glPushAttrib(GL_ALL_ATTRIB_BITS);

  /* apply the appearance if present */
  appearance.render(SFRI);

  /* draw the geometry */
  geometry.render(SFRI);

  /* save the state before rendering */
  glPopAttrib();

#ifdef DEBUG_RENDER
  cout << "\t" << gluErrorString(glGetError()) << "\n";  
  cout << "\tDone Shape\n";
#endif
  
}
/*************************************************************************/

/*************************************************************************/
void Shape::eventIn(SFString &eventType, SFNode *eventData)
{

#ifdef DEBUG_EVENTS
  cout << "\nEntered Shape::eventIn (" << DEF << ")\n";
#endif 
  
  /* Determine what action to take based on incoming event */
  if ((eventType == "appearance") || (eventType == "set_appearance")) {
    Appearance *n = (Appearance *) eventData;
    appearance = *n;

  } else if ((eventType == "geometry") || (eventType == "set_geometry")) {
    Geometry *n = (Geometry *) eventData;
    geometry = *n;
    
  }
}
/*************************************************************************/

/*************************************************************************/
void Shape::parse(char *header, istream &InFile)
{

  /* we are valid */
  isValid() = true;

  /* a token in the file */
  SFToken Token;

  /* the required labels */
  bool TokenFound  = false;
  bool BeginBracketFound   = false;

  /* commence to search for the matching bracket */
  while (! InFile.eof()) {
    
    /* Remember where the token started from */
    Token.GetToken(InFile);

#ifdef DEBUG_PARSE
    cout << "\tShape: (" << Token() << ")\n";
#endif
    
    /* based on what token was found, continue parsing */
    if (Token == "DEF") {
      Token.GetToken(InFile);
      DEF = Token();

    } else if (Token == "USE") {
      Token.GetToken(InFile);
      USE = Token();
      unsatisfiedUSE() = true;
      return;

    } else if (Token == "geometry") {
      geometry.parse(header, InFile);

    } else if (Token == "appearance") { 
      appearance.parse(header, InFile);

    } else if (Token == "Shape") {
      TokenFound = true;

    } else if (Token == "{") {
      BeginBracketFound = true;

    } else if (Token == "}") {
      break;
      
    } else {
      parseWarning(Token());

    }
  }

  /* if we did not find the material token we are in trouble */
  if (TokenFound == false) {
    cerr << "\nError:\n";
    cerr << "\tOccurred in (" << nodeType() << "::parse()\n";
    cerr << "\tDid not find expected identifier token.\n";
    exit(0);
  }
  if (BeginBracketFound == false) {
    cerr << "\nError:\n";
    cerr << "\tOccurred in (" << nodeType() << "::parse()\n";
    cerr << "\tDid not find expected \"{\" token.\n";
    exit(0);
  }
}
/*************************************************************************/
